@@ -27,11 +27,10 @@
 #include "egg/egg-error.h"
 #include "egg/egg-secure-memory.h"
 
-#include <gcr/gcr-unlock-options.h>
-
 #include "pkcs11/pkcs11i.h"
 
-#include "ui/gku-prompt.h"
+#include <gcr/gcr-base.h>
+#include <gcr/gcr-unlock-options.h>
 
 #include <glib/gi18n.h>
 
@@ -51,7 +50,8 @@ EGG_SECURE_DECLARE (gpg_agent_ops);
  */
 
 static void
-keyid_to_field_attribute (const gchar *keyid, GckAttributes *attrs)
+keyid_to_field_attribute (const gchar *keyid,
+                          GckBuilder *attrs)
 {
 	GString *fields = g_string_sized_new (128);
 
@@ -70,7 +70,7 @@ keyid_to_field_attribute (const gchar *keyid, GckAttributes *attrs)
 	g_string_append (fields, "gnome-keyring:gpg-agent");
 	g_string_append_c (fields, '\0');
 
-	gck_attributes_add_data (attrs, CKA_G_FIELDS, (const guchar *)fields->str, fields->len);
+	gck_builder_add_data (attrs, CKA_G_FIELDS, (const guchar *)fields->str, fields->len);
 	g_string_free (fields, TRUE);
 }
 
@@ -111,29 +111,26 @@ calculate_label_for_key (const gchar *keyid, const gchar *description)
 static GList*
 find_saved_items (GckSession *session, GckAttributes *attrs)
 {
-	GckAttributes *template;
+	GckBuilder builder = GCK_BUILDER_INIT;
 	GError *error = NULL;
-	GckAttribute *attr;
+	const GckAttribute *attr;
 	GckObject *search;
 	GList *results;
 	gpointer data;
 	gsize n_data;
 
-	template = gck_attributes_new ();
-	gck_attributes_add_ulong (template, CKA_CLASS, CKO_G_SEARCH);
-	gck_attributes_add_boolean (template, CKA_TOKEN, FALSE);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_G_SEARCH);
+	gck_builder_add_boolean (&builder, CKA_TOKEN, FALSE);
 
 	attr = gck_attributes_find (attrs, CKA_G_COLLECTION);
 	if (attr != NULL)
-		gck_attributes_add (template, attr);
+		gck_builder_add_owned (&builder, attr);
 
 	attr = gck_attributes_find (attrs, CKA_G_FIELDS);
 	g_return_val_if_fail (attr != NULL, NULL);
-	gck_attributes_add (template, attr);
-
-	search = gck_session_create_object (session, template, NULL, &error);
-	gck_attributes_unref (template);
+	gck_builder_add_owned (&builder, attr);
 
+	search = gck_session_create_object (session, gck_builder_end (&builder), NULL, &error);
 	if (search == NULL) {
 		g_warning ("couldn't perform search for gpg agent stored passphrases: %s",
 		           egg_error_message (error));
@@ -162,6 +159,7 @@ static void
 do_save_password (GckSession *session, const gchar *keyid, const gchar *description,
                   const gchar *password, GckAttributes *options)
 {
+	GckBuilder builder;
 	GckAttributes *attrs;
 	gpointer identifier;
 	gsize n_identifier;
@@ -179,23 +177,24 @@ do_save_password (GckSession *session, const gchar *keyid, const gchar *descript
 		return;
 
 	/* Sending a password, needs to be secure */
-	attrs = gck_attributes_new_full (egg_secure_realloc);
+	gck_builder_init_full (&builder, GCK_BUILDER_SECURE_MEMORY);
 
 	/* Build up basic set of attributes */
-	gck_attributes_add_boolean (attrs, CKA_TOKEN, TRUE);
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_SECRET_KEY);
-	keyid_to_field_attribute (keyid, attrs);
+	gck_builder_add_boolean (&builder, CKA_TOKEN, TRUE);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_SECRET_KEY);
+	keyid_to_field_attribute (keyid, &builder);
 
 	/* Bring in all the unlock options */
 	for (i = 0; options && i < gck_attributes_count (options); ++i)
-		gck_attributes_add (attrs, gck_attributes_at (options, i));
+		gck_builder_add_owned (&builder, gck_attributes_at (options, i));
 
 	/* Find a previously stored object like this, and replace if so */
+	attrs = gck_attributes_ref_sink (gck_builder_end (&builder));
 	previous = find_saved_items (session, attrs);
 	if (previous) {
 		identifier = gck_object_get_data (previous->data, CKA_ID, NULL, &n_identifier, NULL);
 		if (identifier != NULL)
-			gck_attributes_add_data (attrs, CKA_ID, identifier, n_identifier);
+			gck_builder_add_data (&builder, CKA_ID, identifier, n_identifier);
 		g_free (identifier);
 		gck_list_unref_free (previous);
 	}
@@ -205,11 +204,13 @@ do_save_password (GckSession *session, const gchar *keyid, const gchar *descript
 	g_free (text);
 
 	/* Put in the remainder of the attributes */
-	gck_attributes_add_string (attrs, CKA_VALUE, password);
-	gck_attributes_add_string (attrs, CKA_LABEL, label);
+	gck_builder_add_all (&builder, attrs);
+	gck_builder_add_string (&builder, CKA_VALUE, password);
+	gck_builder_add_string (&builder, CKA_LABEL, label);
+	gck_attributes_unref (attrs);
 	g_free (label);
 
-	item = gck_session_create_object (session, attrs, NULL, &error);
+	item = gck_session_create_object (session, gck_builder_end (&builder), NULL, &error);
 	if (item == NULL) {
 		g_warning ("couldn't store gpg agent password: %s", egg_error_message (error));
 		g_clear_error (&error);
@@ -217,20 +218,20 @@ do_save_password (GckSession *session, const gchar *keyid, const gchar *descript
 
 	if (item != NULL)
 		g_object_unref (item);
-	gck_attributes_unref (attrs);
 }
 
 static gboolean
 do_clear_password (GckSession *session, const gchar *keyid)
 {
+	GckBuilder builder = GCK_BUILDER_INIT;
 	GckAttributes *attrs;
 	GList *objects, *l;
 	GError *error = NULL;
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_SECRET_KEY);
-	keyid_to_field_attribute (keyid, attrs);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_SECRET_KEY);
+	keyid_to_field_attribute (keyid, &builder);
 
+	attrs = gck_attributes_ref_sink (gck_builder_end (&builder));
 	objects = find_saved_items (session, attrs);
 	gck_attributes_unref (attrs);
 
@@ -255,6 +256,7 @@ do_clear_password (GckSession *session, const gchar *keyid)
 static gchar*
 do_lookup_password (GckSession *session, const gchar *keyid)
 {
+	GckBuilder builder = GCK_BUILDER_INIT;
 	GckAttributes *attrs;
 	GList *objects, *l;
 	GError *error = NULL;
@@ -264,10 +266,10 @@ do_lookup_password (GckSession *session, const gchar *keyid)
 	if (keyid == NULL)
 		return NULL;
 
-	attrs = gck_attributes_new ();
-	gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_SECRET_KEY);
-	keyid_to_field_attribute (keyid, attrs);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_SECRET_KEY);
+	keyid_to_field_attribute (keyid, &builder);
 
+	attrs = gck_attributes_ref_sink (gck_builder_end (&builder));
 	objects = find_saved_items (session, attrs);
 	gck_attributes_unref (attrs);
 
@@ -293,11 +295,11 @@ do_lookup_password (GckSession *session, const gchar *keyid)
 }
 
 static void
-load_unlock_options (GkuPrompt *prompt)
+load_unlock_options (GcrPrompt *prompt)
 {
 	GSettings *settings;
 	gchar *method;
-	gint ttl;
+	gboolean chosen;
 
 	settings = gkd_gpg_agent_settings ();
 
@@ -314,77 +316,68 @@ load_unlock_options (GkuPrompt *prompt)
 		method = g_strdup (GCR_UNLOCK_OPTION_SESSION);
 	}
 
-	gku_prompt_set_unlock_choice (prompt, method);
-	g_free (method);
+	chosen = g_str_equal (GCR_UNLOCK_OPTION_ALWAYS, method);
+	gcr_prompt_set_choice_chosen (prompt, chosen);
 
-	ttl = g_settings_get_int (settings, "gpg-cache-ttl");
-	gku_prompt_set_unlock_ttl (prompt, ttl <= 0 ? 1 : (guint)ttl);
+	g_free (method);
 }
 
 static void
-save_unlock_options (GkuPrompt *prompt)
+save_unlock_options (GcrPrompt *prompt)
 {
 	GSettings *settings;
-	const gchar *method;
-	gint ttl;
 
 	settings = gkd_gpg_agent_settings ();
 
-	method = gku_prompt_get_unlock_choice (prompt);
-	if (method)
-		g_settings_set_string (settings, "gpg-cache-method", method);
-
-	ttl = gku_prompt_get_unlock_ttl (prompt);
-	if (ttl >= 0)
-		g_settings_set_int (settings, "gpg-cache-ttl", (gint)ttl);
+	if (gcr_prompt_get_choice_chosen (prompt))
+		g_settings_set_string (settings, "gpg-cache-method", GCR_UNLOCK_OPTION_ALWAYS);
 }
 
-static GkuPrompt*
-prepare_password_prompt (GckSession *session, const gchar *keyid, const gchar *errmsg,
-                         const gchar *prompt_text, const gchar *description, gboolean confirm)
+static GcrPrompt *
+open_password_prompt (GckSession *session,
+                      const gchar *keyid,
+                      const gchar *errmsg,
+                      const gchar *prompt_text,
+                      const gchar *description,
+                      gboolean confirm)
 {
-	GkuPrompt *prompt;
+	GckBuilder builder = GCK_BUILDER_INIT;
+	GcrPrompt *prompt;
 	GError *error = NULL;
 	gboolean auto_unlock;
 	GList *objects;
-	GckAttributes *attrs;
+	const gchar *choice;
 
 	g_assert (GCK_IS_SESSION (session));
 
-	prompt = gku_prompt_new ();
+	prompt = GCR_PROMPT (gcr_system_prompt_open (-1, NULL, &error));
+	if (prompt == NULL) {
+		g_warning ("couldn't create prompt for gnupg passphrase: %s", egg_error_message (error));
+		g_error_free (error);
+		return NULL;
+	}
 
-	gku_prompt_set_title (prompt, _("Enter Passphrase"));
-	gku_prompt_set_primary_text (prompt, prompt_text ? prompt_text : _("Enter Passphrase"));
-	gku_prompt_set_secondary_text (prompt, description);
+	gcr_prompt_set_title (prompt, _("Enter Passphrase"));
+	gcr_prompt_set_message (prompt, prompt_text ? prompt_text : _("Enter Passphrase"));
+	gcr_prompt_set_description (prompt, description);
 
-	gku_prompt_hide_widget (prompt, "name_area");
-	if (confirm)
-		gku_prompt_show_widget (prompt, "confirm_area");
-	else
-		gku_prompt_hide_widget (prompt, "confirm_area");
-	gku_prompt_show_widget (prompt, "password_area");
+	gcr_prompt_set_password_new (prompt, confirm);
 
 	if (errmsg)
-		gku_prompt_set_warning (prompt, errmsg);
+		gcr_prompt_set_warning (prompt, errmsg);
 
 	if (keyid == NULL) {
-		gku_prompt_hide_widget (prompt, "details_area");
-		gku_prompt_hide_widget (prompt, "options_area");
+		gcr_prompt_set_choice_label (prompt, NULL);
 
 	} else {
-		gku_prompt_show_widget (prompt, "details_area");
-		gku_prompt_show_widget (prompt, "options_area");
-
 		auto_unlock = FALSE;
 
-		attrs = gck_attributes_new ();
-		gck_attributes_add_ulong (attrs, CKA_CLASS, CKO_G_COLLECTION);
-		gck_attributes_add_string (attrs, CKA_ID, "login");
-		gck_attributes_add_boolean (attrs, CKA_G_LOCKED, FALSE);
+		gck_builder_add_ulong (&builder, CKA_CLASS, CKO_G_COLLECTION);
+		gck_builder_add_string (&builder, CKA_ID, "login");
+		gck_builder_add_boolean (&builder, CKA_G_LOCKED, FALSE);
 
 		/* Check if the login keyring is usable */
-		objects = gck_session_find_objects (session, attrs, NULL, &error);
-		gck_attributes_unref (attrs);
+		objects = gck_session_find_objects (session, gck_builder_end (&builder), NULL, &error);
 
 		if (error) {
 			g_warning ("gpg agent couldn't lookup for login keyring: %s", egg_error_message (error));
@@ -395,10 +388,10 @@ prepare_password_prompt (GckSession *session, const gchar *keyid, const gchar *e
 
 		gck_list_unref_free (objects);
 
-		gku_prompt_set_unlock_sensitive (prompt, GCR_UNLOCK_OPTION_ALWAYS, auto_unlock, NULL);
-		gku_prompt_set_unlock_label (prompt, GCR_UNLOCK_OPTION_IDLE, _("Forget this password if idle for"));
-		gku_prompt_set_unlock_label (prompt, GCR_UNLOCK_OPTION_TIMEOUT, _("Forget this password after"));
-		gku_prompt_set_unlock_label (prompt, GCR_UNLOCK_OPTION_SESSION, _("Forget this password when I log out"));
+		choice = NULL;
+		if (auto_unlock)
+			choice = _("Automatically unlock this key, whenever I'm logged in");
+		gcr_prompt_set_choice_label (prompt, choice);
 
 		load_unlock_options (prompt);
 	}
@@ -406,22 +399,16 @@ prepare_password_prompt (GckSession *session, const gchar *keyid, const gchar *e
 	return prompt;
 }
 
-static GkuPrompt*
-on_prompt_attention (gpointer user_data)
-{
-	/* We passed the prompt as the argument */
-	return g_object_ref (user_data);
-}
-
 static gchar*
 do_get_password (GckSession *session, const gchar *keyid, const gchar *errmsg,
                  const gchar *prompt_text, const gchar *description, gboolean confirm)
 {
+	GckBuilder builder = GCK_BUILDER_INIT;
 	GckAttributes *attrs;
 	gchar *password = NULL;
-	GkuPrompt *prompt;
-	const gchar *choice;
-	guint ttl;
+	GcrPrompt *prompt;
+	gboolean chosen;
+	GError *error = NULL;
 
 	g_assert (GCK_IS_SESSION (session));
 
@@ -430,43 +417,36 @@ do_get_password (GckSession *session, const gchar *keyid, const gchar *errmsg,
 	if (password != NULL)
 		return password;
 
-	prompt = prepare_password_prompt (session, keyid, errmsg, prompt_text,
-	                                  description, confirm);
-
-	gku_prompt_request_attention_sync (NULL, on_prompt_attention,
-	                                   g_object_ref (prompt), g_object_unref);
-
-	if (gku_prompt_get_response (prompt) == GKU_RESPONSE_OK) {
-		password = gku_prompt_get_password (prompt, "password");
-		g_return_val_if_fail (password, NULL);
-
-		if (keyid != NULL) {
-			attrs = gck_attributes_new ();
-
-			/* Load up the save options */
-			choice = gku_prompt_get_unlock_choice (prompt);
-			ttl = gku_prompt_get_unlock_ttl (prompt);
-
-			if (g_str_equal (choice, GCR_UNLOCK_OPTION_ALWAYS))
-				gck_attributes_add_string (attrs, CKA_G_COLLECTION, "login");
-			else
-				gck_attributes_add_string (attrs, CKA_G_COLLECTION, "session");
-
-			if (g_str_equal (choice, GCR_UNLOCK_OPTION_IDLE))
-				gck_attributes_add_ulong (attrs, CKA_G_DESTRUCT_IDLE, ttl);
-
-			else if (g_str_equal (choice, GCR_UNLOCK_OPTION_TIMEOUT))
-				gck_attributes_add_ulong (attrs, CKA_G_DESTRUCT_AFTER, ttl);
-
-			/* Now actually save the password */
-			do_save_password (session, keyid, description, password, attrs);
-			gck_attributes_unref (attrs);
-
-			save_unlock_options (prompt);
+	prompt = open_password_prompt (session, keyid, errmsg, prompt_text,
+	                               description, confirm);
+	if (prompt != NULL) {
+		password = egg_secure_strdup (gcr_prompt_password (prompt, NULL, &error));
+		if (password == NULL) {
+			if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+				g_warning ("couldn't prompt for password: %s", egg_error_message (error));
+			g_clear_error (&error);
 		}
 	}
 
-	g_object_unref (prompt);
+	if (password != NULL && keyid != NULL) {
+
+		/* Load up the save options */
+		chosen = gcr_prompt_get_choice_chosen (prompt);
+
+		if (chosen)
+			gck_builder_add_string (&builder, CKA_G_COLLECTION, "login");
+		else
+			gck_builder_add_string (&builder, CKA_G_COLLECTION, "session");
+
+		/* Now actually save the password */
+		attrs = gck_attributes_ref_sink (gck_builder_end (&builder));
+		do_save_password (session, keyid, description, password, attrs);
+		gck_attributes_unref (attrs);
+
+		save_unlock_options (prompt);
+	}
+
+	g_clear_object (&prompt);
 	return password;
 }
 
